#!/usr/bin/python
# -*- coding: utf-8 -*-
######################################
# File name : model.py
# Create date : 2019-10-20 19:47
# Modified date : 2020-04-22 04:37
# Author : liuzy
# Describe : not set
# Email : lzygzh@126.com
######################################

import torch
import torch.nn as nn
import torch.nn.functional as F
import numpy as np
# from mozi_utils import pylog
from mozi_utils import pylog

EPS = 0.003


def fanin_init(size, fanin=None):
    fain = fanin or size[0]
    v = 1. / np.sqrt(256)
    return torch.Tensor(size).uniform_(-v, v)


class Critic(nn.Module):
    """价值网络"""

    def __init__(self, state_dim, action_dim):
        super(Critic, self).__init__()
        self.state_dim = state_dim
        self.action_dim = action_dim

        self.fcs1 = nn.Linear(state_dim, 256)
        self.fcs1.weight.data = fanin_init(self.fcs1.weight.data.size())

        self.fcs2 = nn.Linear(256, 128)
        self.fcs2.weight.data = fanin_init(self.fcs2.weight.data.size())

        self.fca1 = nn.Linear(action_dim, 128)
        self.fca1.weight.data = fanin_init(self.fca1.weight.data.size())

        self.fc2 = nn.Linear(256, 128)
        self.fc2.weight.data = fanin_init(self.fc2.weight.data.size())

        self.fc3 = nn.Linear(128, 1)
        self.fc3.weight.data.uniform_(-EPS, EPS)

    # 正向传播
    def forward(self, state, action):
        s1 = F.relu(self.fcs1(state))
        s2 = F.relu(self.fcs2(s1))
        a1 = F.relu(self.fca1(action))
        x = torch.cat((s2, a1), dim=1)
        x = F.relu(self.fc2(x))
        x = self.fc3(x)
        return x


class Actor(nn.Module):
    """策略网络，四层网络"""

    def __init__(self, state_dim, action_dim, action_lim):
        super(Actor, self).__init__()
        self.state_dim = state_dim
        self.action_dim = action_dim
        self.action_lim = action_lim

        # 全连接层
        self.fc1 = nn.Linear(state_dim, 256)
        # pylog.info(self.fc1.weight.data.size())
        self.fc1.weight.data = fanin_init(self.fc1.weight.data.size())   # initialization of FC1
        # self.fc1.weight.data.normal_(0, 0.1) 服从(0，0.1)的正态分布
        # 全连接层
        self.fc2 = nn.Linear(256, 128)
        self.fc2.weight.data = fanin_init(self.fc2.weight.data.size())

        # 全连接层
        self.fc3 = nn.Linear(128, 64)
        self.fc3.weight.data = fanin_init(self.fc3.weight.data.size())

        # 全连接层
        self.fc4 = nn.Linear(64, action_dim)
        self.fc4.weight.data.uniform_(-EPS, EPS)

    def forward(self, state):
        """正向传播"""
        x = F.relu(self.fc1(state))
        x = F.relu(self.fc2(x))
        x = F.relu(self.fc3(x))
        action = F.tanh(self.fc4(x))
        action = action * self.action_lim   # action range
        return action
